@@ -2,10 +2,10 @@
    Contributors: Nicola Zonta
    Copyright Schrodinger, LLC. All rights reserved
  */
+#pragma once
+#include "CoordgenConfig.hpp"
 
-#ifndef COORDGEN_FRAGMENTER_H
-#define COORDGEN_FRAGMENTER_H
-
+#include <cstddef>
 #include <vector>
 
 class sketcherMinimizerFragment;
@@ -17,7 +17,7 @@ class sketcherMinimizerAtom;
 /*
  class to divide a molecule into rigid fragments
  */
-class CoordgenFragmenter
+class EXPORT_COORDGEN CoordgenFragmenter
 {
   public:
     /*
@@ -65,37 +65,6 @@ class CoordgenFragmenter
     initializeInformation(std::vector<sketcherMinimizerFragment*> fragments,
                           sketcherMinimizerMolecule* molecule);
 
-    /*
-     check if fragment is part of an aliphatic chain and sets the appropriate
-     flag
-     */
-    static void setChainInfo(sketcherMinimizerFragment* fragment);
-
-    /*
-     check if fragment has fixed coordinates and sets the appropriate flag
-     */
-    static bool setFixedInfo(sketcherMinimizerFragment* fragment);
-
-    /*
-     check if fragment has constrained coordinates and set the appropriate flag
-     */
-    static bool setConstrainedInfo(sketcherMinimizerFragment* fragment);
-
-    /*
-     check if atom has fixed cooordinates
-     */
-    static bool isAtomFixed(const sketcherMinimizerAtom* atom);
-
-    /*
-     check if atom has constrained cooordinates
-     */
-    static bool isAtomConstrained(const sketcherMinimizerAtom* atom);
-
-    /*
-     check if fragment is part of an aliphatic chain
-     */
-    static bool isChain(const sketcherMinimizerFragment* fragment);
-
     /*
       compare two fragments for priority (e.g. to find the main fragment)
      */
@@ -106,21 +75,21 @@ class CoordgenFragmenter
      get the score of a particular descriptor for the given fragment (used to
      assign priorities)
      */
-    static int getValueOfCheck(const sketcherMinimizerFragment* fragment,
-                               int checkN, bool& checkNoMore);
+    static size_t getValueOfCheck(const sketcherMinimizerFragment* fragment,
+                                  int checkN, bool& checkNoMore);
 
     /*
      find the main fragment
      */
     static sketcherMinimizerFragment*
-    findMainFragment(std::vector<sketcherMinimizerFragment*> fragments);
+    findMainFragment(const std::vector<sketcherMinimizerFragment*>& fragments);
 
     /*
      if molecule has a long enough chain return the start of the chain as main
      fragment instead
      */
     static sketcherMinimizerFragment*
-    considerChains(std::vector<sketcherMinimizerFragment*> fragments,
+    considerChains(const std::vector<sketcherMinimizerFragment*>& fragments,
                    sketcherMinimizerFragment* mainFragment);
 
     /* empirical minimum length of zigzag chain of fragments
@@ -134,14 +103,14 @@ class CoordgenFragmenter
      find the longest chain of connected fragments
      */
     static std::vector<sketcherMinimizerFragment*>
-    findLongestChain(std::vector<sketcherMinimizerFragment*> fragments);
+    findLongestChain(const std::vector<sketcherMinimizerFragment*>& fragments);
 
     /*
      initialize parent-child info of bound fragments
      */
     static void addParentRelationsToFragments(
         sketcherMinimizerFragment* mainFragment,
-        std::vector<sketcherMinimizerFragment*> fragments);
+        const std::vector<sketcherMinimizerFragment*>& fragments);
 
     /*
      order the vector of fragments so that bound fragments are consecutive,
@@ -152,5 +121,3 @@ class CoordgenFragmenter
     orderFragments(std::vector<sketcherMinimizerFragment*>& fragments,
                    sketcherMinimizerFragment* mainFragment);
 };
-
-#endif /* defined(COORDGEN_FRAGMENTER_H) */
