@@ -2,14 +2,16 @@
  Contributors: Nicola Zonta
  Copyright Schrodinger, LLC. All rights reserved
  */
+#include <algorithm>
+#include <queue>
 
 #include "CoordgenMacrocycleBuilder.h"
 #include "CoordgenMinimizer.h"
 #include "sketcherMinimizer.h"
 #include "sketcherMinimizerMaths.h"
+#include "sketcherMinimizerRing.h"
 #include "sketcherMinimizerStretchInteraction.h"
-#include <algorithm>
-#include <queue>
+
 
 #define MAX_MACROCYCLES 40
 #define PATH_FAILED -1000
@@ -719,14 +721,13 @@ sketcherMinimizerBond*
 CoordgenMacrocycleBuilder::findBondToOpen(sketcherMinimizerRing* ring) const
 {
     sketcherMinimizerBond* bestBond = nullptr;
-    size_t bestScore = 0;
+    float bestScore = 0.f;
     for (sketcherMinimizerBond* bond : ring->_bonds) {
-        size_t score = 0;
+        float score = 0.f;
         if (ring->isMacrocycle()) {
             if (bond->getBondOrder() != 1) {
                 continue;
             }
-
             bool nextToMultipleBond = false;
             for (auto otherBond : bond->getStartAtom()->bonds) {
                 if (otherBond->isStereo()) {
@@ -747,6 +748,7 @@ CoordgenMacrocycleBuilder::findBondToOpen(sketcherMinimizerRing* ring) const
         score += bond->rings.size() * 10;
         score += bond->getStartAtom()->neighbors.size();
         score += bond->getEndAtom()->neighbors.size();
+        score /= bond->crossingBondPenaltyMultiplier;
         if (bestBond == nullptr || score < bestScore) {
             bestScore = score;
             bestBond = bond;
@@ -761,8 +763,8 @@ bool CoordgenMacrocycleBuilder::openCycleAndGenerateCoords(
 {
     map<sketcherMinimizerAtom*, sketcherMinimizerAtom*> atomMap;
     sketcherMinimizer min(getPrecision());
-    min.m_minimizer.skipMinimization = true;
-    min.m_fragmentBuilder.setForceOpenMacrocycles(true);
+    min.setSkipMinimization(true);
+    min.setForceOpenMacrocycles(true);
     auto* minMol = new sketcherMinimizerMolecule;
     sketcherMinimizerBond* bondToBreak = findBondToOpen(ring);
     if (!bondToBreak) {
@@ -803,7 +805,7 @@ bool CoordgenMacrocycleBuilder::openCycleAndGenerateCoords(
     }
     min.initialize(minMol);
     min.findFragments();
-    min.m_minimizer.buildFromFragments(true);
+    min.buildFromFragments(true);
     auto* brokenBond = new sketcherMinimizerBond;
     brokenBond->bondOrder = bondToBreak->bondOrder;
     brokenBond->startAtom = atomMap[bondToBreak->startAtom];
@@ -814,7 +816,7 @@ bool CoordgenMacrocycleBuilder::openCycleAndGenerateCoords(
     std::vector<sketcherMinimizerInteraction*> extraInteractions;
     extraInteractions.push_back(new sketcherMinimizerStretchInteraction(
         brokenBond->startAtom, brokenBond->endAtom));
-    min.m_minimizer.avoidClashesOfMolecule(minMol, extraInteractions);
+    min.avoidClashesOfMolecule(minMol, extraInteractions);
 
     for (sketcherMinimizerAtom* atom : atoms) {
         sketcherMinimizerAtom* otherAtom = atomMap[atom];
