@@ -18,6 +18,7 @@
 #include "sketcherMinimizerRing.h"
 #include "sketcherMinimizerStretchInteraction.h"
 #include <algorithm>
+#include <limits>
 #include <queue>
 
 using namespace std;
@@ -35,14 +36,20 @@ static const float RING_BOND_CROSSING_MULTIPLIER = 2.f;
 
 static const unsigned int MAXIMUM_NUMBER_OF_SCORED_SOLUTIONS = 10000;
 static const float REJECTED_SOLUTION_SCORE = 99999999.f;
+
+static const unsigned int ITERATION_HISTORY_SIZE = 100;
+static const float MAX_NET_ENERGY_CHANGE = 20.f;
+
 CoordgenMinimizer::CoordgenMinimizer()
 {
-    m_maxIterations = 500;
+    m_maxIterations = 1000;
     skipMinimization = false;
     skipFlipFragments = false;
     skipAvoidClashes = false;
     m_scoreResidueInteractions = true;
     m_precision = 1.f;
+    energy_list = {};
+    all_coordinates = {};
 }
 
 CoordgenMinimizer::~CoordgenMinimizer()
@@ -71,20 +78,55 @@ void CoordgenMinimizer::run()
         setupInteractions();
     }
 
-    for (int iterations = 0; iterations < m_maxIterations; ++iterations) {
-        scoreInteractions();
-        if (!applyForces()) {
+#ifdef DEBUG_MINIMIZATION_COORDINATES
+    // to seperate energy and DOF minimization
+    energy_list.push_back(-1.f);
+    all_coordinates.push_back({sketcherMinimizerPointF(0,0)});
+#endif
+
+    std::vector<float> local_energy_list(m_maxIterations);
+    std::vector<sketcherMinimizerPointF> lowest_energy_coords(m_atoms.size());
+    float min_energy = std::numeric_limits<float>::max();
+    for (unsigned int iterations = 0; iterations < m_maxIterations; ++iterations) {
+        local_energy_list[iterations] = scoreInteractions();
+        // track coordinates with lowest energy
+        if (local_energy_list[iterations] < min_energy) {
+            for (size_t i = 0; i < m_atoms.size(); ++i) {
+                lowest_energy_coords[i] = m_atoms[i]->coordinates;
+            }
+        }
+#ifdef DEBUG_MINIMIZATION_COORDINATES
+        // store data from this minimization step to be written to a file later
+        energy_list.push_back(local_energy_list[iterations]);
+        std::vector<sketcherMinimizerPointF> these_coordinates;
+        for (auto atom : m_atoms) {
+            these_coordinates.push_back(atom->coordinates);
+        }
+        all_coordinates.push_back(these_coordinates);
+#endif
+        if (!applyForces(0.1f)) {
+            break;
+        }
+        if (iterations < 2 * ITERATION_HISTORY_SIZE) {
+            continue;
+        }
+        if (local_energy_list[iterations - ITERATION_HISTORY_SIZE] - local_energy_list[iterations] < MAX_NET_ENERGY_CHANGE) {
             break;
         }
     }
-    fixRingsShape();
+    // set coordinates back to lowest energy state
+    if (min_energy < std::numeric_limits<float>::max()) {
+        for (size_t i = 0; i < m_atoms.size(); ++i) {
+            m_atoms[i]->coordinates = lowest_energy_coords[i];
+        }
+    }
 }
 
 bool CoordgenMinimizer::applyForces(float maxd)
 {
     float delta = 0.001f; // minimum squared displacement
     float distance = 0.f;
-    for (auto atom : _atoms) {
+    for (auto atom : m_atoms) {
         if (atom->fixed) {
             continue;
         }
@@ -213,6 +255,10 @@ void CoordgenMinimizer::addStretchInteractionsOfMolecule(
             sketcherMinimizerPointF v = at2->coordinates - at1->coordinates;
             interaction->restV = v.length();
         }
+        auto sharedRing = sketcherMinimizer::sameRing(at1, at2);
+        if (sharedRing && !sharedRing->isMacrocycle()) {
+            interaction->k *= 50;
+        }
         _interactions.push_back(interaction);
         _stretchInteractions.push_back(interaction);
     }
@@ -221,7 +267,7 @@ void CoordgenMinimizer::addStretchInteractionsOfMolecule(
 /* return a set of all carbons part of a carbonyl group, i.e. doubly bonded to
  * an oxygen. */
 std::set<sketcherMinimizerAtom*> CoordgenMinimizer::getChetoCs(
-    const std::vector<sketcherMinimizerAtom*>& allAtoms)
+    const std::vector<sketcherMinimizerAtom*>& allAtoms) const
 {
     std::set<sketcherMinimizerAtom*> chetoCs;
     for (auto atom : allAtoms) {
@@ -244,7 +290,7 @@ std::set<sketcherMinimizerAtom*> CoordgenMinimizer::getChetoCs(
 /* return a set of all amino nitrogens. Not chemically accurate, doesn't filter
  * out nitro Ns for instance. */
 std::set<sketcherMinimizerAtom*> CoordgenMinimizer::getAminoNs(
-    const std::vector<sketcherMinimizerAtom*>& allAtoms)
+    const std::vector<sketcherMinimizerAtom*>& allAtoms) const
 {
     std::set<sketcherMinimizerAtom*> aminoNs;
     for (auto atom : allAtoms) {
@@ -261,7 +307,7 @@ std::set<sketcherMinimizerAtom*> CoordgenMinimizer::getAminoNs(
 std::set<sketcherMinimizerAtom*> CoordgenMinimizer::getAlphaCs(
     const std::vector<sketcherMinimizerAtom*>& allAtoms,
     const std::set<sketcherMinimizerAtom*>& chetoCs,
-    const std::set<sketcherMinimizerAtom*>& aminoNs)
+    const std::set<sketcherMinimizerAtom*>& aminoNs) const
 {
     std::set<sketcherMinimizerAtom*> alphaCs;
     for (auto atom : allAtoms) {
@@ -451,7 +497,7 @@ void CoordgenMinimizer::addBendInteractionsOfMolecule(
                             }
                         }
                         interaction->isRing = true;
-                        interaction->k *= 10;
+                        interaction->k *= 100;
                         interaction->restV = static_cast<float>(
                             180. - (360. / (r->size() + extraAtoms)));
                         ringInteractions.push_back(interaction);
@@ -622,8 +668,8 @@ void CoordgenMinimizer::addInteractionsOfMolecule(
 void CoordgenMinimizer::setupInteractionsOnlyResidues()
 {
     const float CLASH_DISTANCE = bondLength * 1.5f;
-    for (auto res : _residues) {
-        for (auto res2 : _residues) {
+    for (auto res : m_residues) {
+        for (auto res2 : m_residues) {
             if (res2 >= res) {
                 continue;
             }
@@ -665,7 +711,7 @@ void CoordgenMinimizer::setupInteractionsProteinOnly(
 void CoordgenMinimizer::setupInteractions(bool intrafragmentClashes)
 {
     clearInteractions();
-    for (sketcherMinimizerMolecule* molecule : _molecules) {
+    for (sketcherMinimizerMolecule* molecule : m_molecules) {
         addInteractionsOfMolecule(molecule, intrafragmentClashes);
     }
 }
@@ -744,27 +790,6 @@ bool CoordgenMinimizer::findIntermolecularClashes(
     return false;
 }
 
-void CoordgenMinimizer::fixRingsShape()
-{
-    for (auto in : _bendInteractions) {
-        if (in->isRing) {
-            in->k *= 10;
-        }
-    }
-    for (auto in : _stretchInteractions) {
-        if (sketcherMinimizer::sameRing(in->atom1, in->atom2)) {
-            in->k *= 10;
-        }
-    }
-
-    for (int iterations = 0; iterations < m_maxIterations; ++iterations) {
-        scoreInteractions();
-        if (!applyForces(1)) {
-            break;
-        }
-    }
-}
-
 float CoordgenMinimizer::scoreClashes(
     sketcherMinimizerMolecule* molecule, bool residueInteractions,
     bool scoreProximityRelationsOnOppositeSid) const
@@ -838,8 +863,8 @@ float CoordgenMinimizer::scoreCrossBonds(sketcherMinimizerMolecule* molecule,
             }
         }
     }
-    if (_residueInteractions.size() && residueInteractions) {
-        for (auto r : _residues) {
+    if (m_residueInteractions.size() && residueInteractions) {
+        for (auto r : m_residues) {
             if (r->residueInteractions.size() > 1) {
                 for (unsigned int ri1 = 0;
                      ri1 < r->residueInteractions.size() - 1; ri1++) {
@@ -859,7 +884,7 @@ float CoordgenMinimizer::scoreCrossBonds(sketcherMinimizerMolecule* molecule,
                             out += 15.f;
                         }
 
-                        for (auto b2 : _bonds) {
+                        for (auto b2 : m_bonds) {
                             if (b2->startAtom ==
                                 r->residueInteractions[ri1]->endAtom) {
                                 continue;
@@ -896,7 +921,7 @@ float CoordgenMinimizer::scoreAtomsInsideRings() const
 {
     float out = 0.f;
     float cutOff = bondLength;
-    for (sketcherMinimizerMolecule* m : _molecules) {
+    for (sketcherMinimizerMolecule* m : m_molecules) {
         for (sketcherMinimizerRing* r : m->_rings) {
             if (r->_atoms.size() > MACROCYCLE) {
                 continue;
@@ -939,7 +964,7 @@ float CoordgenMinimizer::scoreAtomsInsideRings() const
 float CoordgenMinimizer::scoreProximityRelationsOnOppositeSides() const
 {
     float out = 0.f;
-    for (sketcherMinimizerMolecule* m : _molecules) {
+    for (sketcherMinimizerMolecule* m : m_molecules) {
         if (m->_atoms.size() < 2) {
             continue;
         }
@@ -1148,6 +1173,15 @@ bool CoordgenMinimizer::runSearch(int tier, CoordgenDOFSolutions& solutions)
     int i = 0;
     bool hasValidSolution = true;
     do {
+#ifdef DEBUG_MINIMIZATION_COORDINATES
+        // store data from this minimization step to be written to a file later
+        energy_list.push_back(solutions.scoreCurrentSolution());
+        std::vector<sketcherMinimizerPointF> these_coordinates;
+        for (auto atom : _atoms) {
+            these_coordinates.push_back(atom->coordinates);
+        }
+        all_coordinates.push_back(these_coordinates);
+#endif
         ++i;
         hasValidSolution = growSolutions(
             allScoredSolutions, tier, growingSolutions, solutions, bestScore);
@@ -1249,7 +1283,7 @@ bool CoordgenMinimizer::avoidClashes()
     if (skipAvoidClashes) {
         return true;
     }
-    for (sketcherMinimizerMolecule* molecule : _molecules) {
+    for (sketcherMinimizerMolecule* molecule : m_molecules) {
         auto cleanPose = avoidClashesOfMolecule(molecule);
         allCleanPoses = allCleanPoses && cleanPose;
     }
@@ -1449,7 +1483,7 @@ void CoordgenMinimizer::buildMoleculeFromFragments(
 
 void CoordgenMinimizer::buildFromFragments(bool firstTime) const
 {
-    for (sketcherMinimizerMolecule* molecule : _molecules) {
+    for (sketcherMinimizerMolecule* molecule : m_molecules) {
         buildMoleculeFromFragments(molecule, firstTime);
     }
 }
@@ -1489,7 +1523,7 @@ bool CoordgenMinimizer::hasNaNCoordinates(
 
 bool CoordgenMinimizer::hasNaNCoordinates()
 {
-    return hasNaNCoordinates(_atoms);
+    return hasNaNCoordinates(m_atoms);
 }
 
 void CoordgenMinimizer::checkForClashes(sketcherMinimizerAtom* a)
