@@ -4,8 +4,8 @@
  */
 
 #include "sketcherMinimizerFragment.h"
-#include "sketcherMinimizerBond.h"
 #include "sketcherMinimizerAtom.h"
+#include "sketcherMinimizerBond.h"
 #include "sketcherMinimizerMaths.h"
 #include "sketcherMinimizerRing.h"
 
@@ -16,6 +16,7 @@ static const float ROTATE_FRAGMENT_PENALTY = 400.f;
 static const float BREAK_CHAIN_PENALTY = 10.f;
 static const float CHANGE_PARENT_BOND_PENALTY = 200.f;
 static const float FLIP_RING_PENALTY = 200.f;
+static const float FLIP_CONSTRAINED_FRAGMENT_PENALTY = 1000.f;
 
 static const int FLIP_FRAGMENT_TIER = 0;
 static const int INVERT_BOND_TIER = 1;
@@ -30,9 +31,7 @@ CoordgenFragmentDOF::CoordgenFragmentDOF(sketcherMinimizerFragment* fragment)
 {
 }
 
-CoordgenFragmentDOF::~CoordgenFragmentDOF()
-{
-}
+CoordgenFragmentDOF::~CoordgenFragmentDOF() = default;
 
 short unsigned int CoordgenFragmentDOF::getCurrentState()
 {
@@ -84,17 +83,22 @@ CoordgenFlipFragmentDOF::CoordgenFlipFragmentDOF(
 
 float CoordgenFlipFragmentDOF::getCurrentPenalty() const
 {
+    float penalty = 0.f;
+    if (m_currentState != 0 && m_fragment->constrainedFlip) {
+        penalty += FLIP_CONSTRAINED_FRAGMENT_PENALTY;
+    }
     if (m_fragment->isChain && m_fragment->getParent() &&
         m_fragment->getParent()->isChain) {
-        return BREAK_CHAIN_PENALTY;
+        penalty += BREAK_CHAIN_PENALTY;
     }
-    return 0.f;
+    return penalty;
 }
 
 int CoordgenFlipFragmentDOF::numberOfStates() const
 {
-    if (m_fragment->getParent() == NULL)
+    if (m_fragment->getParent() == nullptr) {
         return 1;
+    }
     return 2;
 }
 
@@ -120,8 +124,9 @@ CoordgenScaleFragmentDOF::CoordgenScaleFragmentDOF(
 
 int CoordgenScaleFragmentDOF::numberOfStates() const
 {
-    if (m_fragment->getRings().size() == 0)
+    if (m_fragment->getRings().size() == 0) {
         return 1;
+    }
     return 5;
 }
 
@@ -133,7 +138,7 @@ int CoordgenScaleFragmentDOF::tier() const
 void CoordgenScaleFragmentDOF::apply() const
 {
     if (m_currentState != 0) {
-        float scale = pow(1.4, (m_currentState + 1) / 2);
+        auto scale = static_cast<float>(pow(1.4, (m_currentState + 1) / 2));
         if (m_currentState % 2 == 0) {
             scale = 1 / scale;
         }
@@ -169,7 +174,7 @@ int CoordgenScaleAtomsDOF::tier() const
 void CoordgenScaleAtomsDOF::apply() const
 {
     if (m_currentState != 0) {
-        float scale = 0.4;
+        float scale = 0.4f;
         for (auto atom : m_atoms) {
             auto distance =
                 atom->getCoordinates() - m_pivotAtom->getCoordinates();
@@ -207,7 +212,7 @@ int CoordgenChangeParentBondLengthFragmentDOF::tier() const
 void CoordgenChangeParentBondLengthFragmentDOF::apply() const
 {
     if (m_currentState != 0) {
-        float scale = pow(1.6, (m_currentState + 1) / 2);
+        auto scale = static_cast<float>(pow(1.6, (m_currentState + 1) / 2));
         if (m_currentState % 2 == 0) {
             scale = 1 / scale;
         }
@@ -234,8 +239,9 @@ CoordgenRotateFragmentDOF::CoordgenRotateFragmentDOF(
 
 int CoordgenRotateFragmentDOF::numberOfStates() const
 {
-    if (m_fragment->getParent() == NULL)
+    if (m_fragment->getParent() == nullptr) {
         return 1;
+    }
     return 5;
 }
 
@@ -247,7 +253,8 @@ int CoordgenRotateFragmentDOF::tier() const
 void CoordgenRotateFragmentDOF::apply() const
 {
     if (m_currentState != 0) {
-        float angle = M_PI / 180 * 15 * ((m_currentState + 1) / 2);
+        auto angle =
+            static_cast<float>(M_PI / 180 * 15 * ((m_currentState + 1) / 2));
         if (m_currentState % 2 == 0) {
             angle = -angle;
         }
@@ -316,14 +323,15 @@ float CoordgenInvertBondDOF::getCurrentPenalty() const
 
 CoordgenFlipRingDOF::CoordgenFlipRingDOF(
     sketcherMinimizerRing* ring,
-    std::vector<sketcherMinimizerAtom*> fusionAtoms)
+    const std::vector<sketcherMinimizerAtom*>& fusionAtoms)
     : CoordgenFragmentDOF((*fusionAtoms.begin())->getFragment()),
       m_pivotAtom1(*(fusionAtoms.begin())),
       m_pivotAtom2(*(fusionAtoms.rbegin())),
       m_penalty(std::abs(int(ring->size() - 2 * fusionAtoms.size() + 2)))
 {
-    for (auto atom : ring->getAtoms())
+    for (auto atom : ring->getAtoms()) {
         addAtom(atom);
+    }
 }
 
 int CoordgenFlipRingDOF::numberOfStates() const
@@ -357,8 +365,9 @@ float CoordgenFlipRingDOF::getCurrentPenalty() const
 
 sketcherMinimizerFragment::sketcherMinimizerFragment()
     : fixed(false), isTemplated(false), constrained(false), isChain(false),
-      _bondToParent(NULL), longestChainFromHere(0.f), numberOfChildrenAtoms(0),
-      numberOfChildrenAtomsRank(0.f), m_parent(NULL)
+      _bondToParent(nullptr), longestChainFromHere(0.f),
+      numberOfChildrenAtoms(0), numberOfChildrenAtomsRank(0.f),
+      m_parent(nullptr)
 {
     m_dofs.push_back(new CoordgenFlipFragmentDOF(this));
     //    m_dofs.push_back(new CoordgenScaleFragmentDOF(this));
@@ -378,7 +387,7 @@ void sketcherMinimizerFragment::addDof(CoordgenFragmentDOF* dof)
     m_dofs.push_back(dof);
 }
 
-std::vector<CoordgenFragmentDOF*> sketcherMinimizerFragment::getDofs()
+const std::vector<CoordgenFragmentDOF*>& sketcherMinimizerFragment::getDofs()
 {
     return m_dofs;
 }
@@ -386,24 +395,29 @@ std::vector<CoordgenFragmentDOF*> sketcherMinimizerFragment::getDofs()
 unsigned int sketcherMinimizerFragment::totalWeight() const
 {
     int n = 0;
-    for (unsigned int i = 0; i < m_atoms.size(); i++)
-        n += m_atoms[i]->atomicNumber + m_atoms[i]->_implicitHs;
+    for (auto m_atom : m_atoms) {
+        n += m_atom->atomicNumber + m_atom->_implicitHs;
+    }
     return n;
-};
+}
 unsigned int sketcherMinimizerFragment::countDoubleBonds() const
 {
     int n = 0;
-    for (unsigned int i = 0; i < m_bonds.size(); i++)
-        if (m_bonds[i]->bondOrder == 2)
+    for (auto m_bond : m_bonds) {
+        if (m_bond->bondOrder == 2) {
             ++n;
+        }
+    }
     return n;
-};
+}
 unsigned int sketcherMinimizerFragment::countHeavyAtoms() const
 {
     int n = 0;
-    for (unsigned int i = 0; i < m_atoms.size(); i++)
-        if (m_atoms[i]->atomicNumber != 6)
+    for (auto m_atom : m_atoms) {
+        if (m_atom->atomicNumber != 6) {
             ++n;
+        }
+    }
     return n;
 }
 
@@ -411,8 +425,9 @@ unsigned int sketcherMinimizerFragment::countConstrainedAtoms() const
 {
     int n = 0;
     for (auto atom : m_atoms) {
-        if (atom->constrained)
+        if (atom->constrained) {
             ++n;
+        }
     }
     return n;
 }
@@ -421,8 +436,9 @@ unsigned int sketcherMinimizerFragment::countFixedAtoms() const
 {
     int n = 0;
     for (auto atom : m_atoms) {
-        if (atom->fixed)
+        if (atom->fixed) {
             ++n;
+        }
     }
     return n;
 }
@@ -443,14 +459,14 @@ void sketcherMinimizerFragment::addRing(sketcherMinimizerRing* ring)
 
 void sketcherMinimizerFragment::setAllCoordinatesToTemplate()
 {
-    foreach (sketcherMinimizerAtom* atom, m_atoms) {
+    for (sketcherMinimizerAtom* atom : m_atoms) {
         atom->setCoordinatesToTemplate();
     }
     if (_bondToParent) {
         _bondToParent->startAtom->setCoordinatesToTemplate();
         _bondToParent->endAtom->setCoordinatesToTemplate();
     }
-    foreach (sketcherMinimizerFragment* child, _children) {
+    for (sketcherMinimizerFragment* child : _children) {
         child->_bondToParent->startAtom->setCoordinatesToTemplate();
         child->_bondToParent->endAtom->setCoordinatesToTemplate();
     }
@@ -467,8 +483,9 @@ void sketcherMinimizerFragment::storeCoordinateInformation()
         angle = atan2(_bondToParent->startAtom->coordinates.y() - origin.y(),
                       -_bondToParent->startAtom->coordinates.x() + origin.x());
     } else {
-        if (!constrained && !fixed)
+        if (!constrained && !fixed) {
             origin = m_atoms[0]->getCoordinates();
+        }
     }
 
     float cosine = cos(-angle);
@@ -488,8 +505,8 @@ void sketcherMinimizerFragment::storeCoordinateInformation()
     }
 }
 
-void sketcherMinimizerFragment::setCoordinates(sketcherMinimizerPointF position,
-                                               float angle)
+void sketcherMinimizerFragment::setCoordinates(
+    const sketcherMinimizerPointF& position, float angle)
 {
     float sine = sin(angle);
     float cosine = cos(angle);
@@ -506,4 +523,4 @@ void sketcherMinimizerFragment::setCoordinates(sketcherMinimizerPointF position,
         initialCoordinates.rotate(sine, cosine);
         atom->setCoordinates(initialCoordinates + position);
     }
-};
+}
